/*============================================================================
  File:     sparse_columns.sql

  Summary:  This script shows how to define and use sparse columns that are 
			supported in SQL Server 2008.

  Date:     August 2008

  SQL Server Version: 10.0.1600.22 (RTM)
------------------------------------------------------------------------------
  Written by Bob Beauchemin, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended only as a supplement to SQL Server Jumpstart program. 
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

if DATABASEPROPERTY('testdb', 'Version') is not null
drop database testdb
go

create database testdb
go

use testdb
go


-- Create a table with sparse properties and column_set 
Create Table Products(
	Id int, Type nvarchar(16),  
	Resolution nvarchar(8) SPARSE, 
    ZoomLength nvarchar(8) SPARSE,
	WaistSize int SPARSE, 
    Length int SPARSE, 			     	
    ProductProperties XML COLUMN_SET FOR ALL_SPARSE_COLUMNS);

-- insert couple of different products
Insert into Products(Id, Type, Resolution, ZoomLength) 
	values (101, 'Camera', '6 mb', '3x');
Insert into Products(Id, Type, WaistSize, Length) 
	values (5001, 'Pant', 32, 32)

-- Access individual product properties
Select Id, Resolution, ZoomLength from Products where Type='camera';

-- Generic XML access to properties through sparse_column_set
-- Returns an aggregated XML fragment for all non-null sparse columns
Select Id, Type, ProductProperties from Products; 

-- Insert a new product through sparse_column_set. 
Declare @xml xml = '<WaistSize>32</WaistSize><Length>32</Length>'
Insert into Products (Id, Type, ProductProperties) 
	      values (5002, 'Pant', @xml);

SET @xml = '<Resolution>10 mb</Resolution><ZoomLength>5x</ZoomLength>'
Update Products set ProductProperties = @xml where Id = 101;

go

-- filtered index on sparse columns
Create Index Ix on Products(ZoomLength) where Type='Camera';
go

select  ZoomLength, *
from Products
where type = 'Camera'
go
-- filtered statistics
Create statistics stx on Products(WaistSize) 
                                      where Type='Pant';
go