/*============================================================================
  File:     insert_over_merge_dml.sql

  Summary:  This script shows that you can use the OUTPUT from the T-SQL
			OUTPUT clause as the input to an insert statement. This construct 
			is also known as "composable DML".

  Date:     August 2008

  SQL Server Version: 10.0.1600.22 (RTM)
------------------------------------------------------------------------------
  Written by Bob Beauchemin, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended only as a supplement to SQL Server Jumpstart program.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE tempdb
GO

IF OBJECTPROPERTY(OBJECT_ID('dbo.Stock'), 'IsTable') = 1
  drop table dbo.Stock
GO
IF OBJECTPROPERTY(OBJECT_ID('dbo.Trades'), 'IsTable') = 1
  drop table dbo.Trades
GO
IF OBJECTPROPERTY(OBJECT_ID('dbo.AuditChanges'), 'IsTable') = 1
  drop table dbo.AuditChanges
GO

CREATE TABLE dbo.Stock (Stock VARCHAR(10) PRIMARY KEY, Qty INT CHECK (Qty > 0));
CREATE TABLE dbo.Trades (Stock VARCHAR(10) PRIMARY KEY, Delta INT CHECK (Delta <> 0));
GO

CREATE TABLE dbo.AuditChanges (Action varchar(6), Stock VARCHAR(6), Qty INT);
GO

INSERT dbo.Stock VALUES('MSFT', 10), ('BOEING', 5);
INSERT dbo.Trades VALUES('MSFT', 5), ('BOEING', -5), ('GE', 3);
GO

INSERT INTO dbo.AuditChanges
SELECT * FROM
(
MERGE dbo.Stock S
  USING dbo.Trades T
  ON S.Stock = T.Stock
  WHEN MATCHED AND (Qty + T.Delta = 0) THEN
    DELETE
  WHEN MATCHED THEN 
    UPDATE SET Qty += T.Delta
  WHEN NOT MATCHED THEN
    INSERT VALUES(Stock, T.Delta);
  OUTPUT $action, T.Stock, inserted.Qty
) tab (action, Stock, qty);
GO


SELECT * FROM dbo.AuditChanges;
GO


