/*============================================================================
  File:     plan_guide_for_update.sql

  Summary:  Illustrates SQL Server 2008 support for plan guides over an 
			UPDATE statement. SQL Server 2005 supported plan guides for SELECT
			only. SQL Server 2008 supports plan guides on INSERT, DELETE, 
			UPDATE, and MERGE statements as well as SELECT.
			
			Also illustrates sp_create_plan_guide_from_handle for easier
			plan guide creation.

  Date:     August 2008

  SQL Server Version: 10.0.1600.22 (RTM)
------------------------------------------------------------------------------
  Written by Bob Beauchemin, SQLskills.com

  For more scripts and sample code, check out http://www.SQLskills.com

  This script is intended only as a supplement to SQL Server Jumpstart program.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

use pubs
go

dbcc freeproccache
go

select * from dbo.titles where type = 'business'
go

create proc dbo.updatetitles (@1 varchar(20))
as
update dbo.titles 
 set price = price * 1.1
 where type = @1;
go

execute dbo.updatetitles 'business'
go

DECLARE @plan_handle varbinary(64);
DECLARE @offset int;
declare @t nvarchar(max)
SELECT @plan_handle = plan_handle, @offset = qs.statement_start_offset,@t = text
FROM sys.dm_exec_query_stats AS qs
CROSS APPLY sys.dm_exec_sql_text(sql_handle) AS st
CROSS APPLY sys.dm_exec_text_query_plan(qs.plan_handle, qs.statement_start_offset, qs.statement_end_offset) AS qp
WHERE text LIKE N'%updatetitles%' and text not like '%plan_handle%'
/*
AND SUBSTRING(st.text, (qs.statement_start_offset/2) + 1,
                        ((CASE statement_end_offset 
                              WHEN -1 THEN DATALENGTH(st.text)
                              ELSE qs.statement_end_offset END 
                              - qs.statement_start_offset)/2) + 1)  like 'SELECT * FROM Production.Product WHERE ProductSubcategoryID > 4%'
*/
select @plan_handle, @t

EXECUTE sp_create_plan_guide_from_handle 
    @name =  N'Guide_For_Update_Statement',
    @plan_handle = @plan_handle,
    @statement_start_offset = @offset;
    
go
sp_control_plan_guide N'enable',N'Guide_For_Update_Statement'

select * from sys.plan_guides
go

-- execute the statement again
-- you can see plan guide usage in new profiler events
execute dbo.updatetitles 'business'
go
